global.__base     = __dirname + '/';
global.__api      = __base + 'routes/api/';
global.__services = __base + 'services/';

const createError  = require('http-errors'),
      compression  = require('compression'),
      express      = require('express'),
      path         = require('path'),
      cookieParser = require('cookie-parser'),
      logger       = require('morgan'),
      isProduction = (process.env.NODE_ENV
          && process.env.NODE_ENV.trim() === 'production') ? true : false;

// Routers
//
const indexRouter   = require('./routes/index'),
      valenceRouter = require('./routes/valence');

// Express
//
const app = express();

// view engine setup
//
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'pug');

// setup environment
//
if (isProduction) {
    app.use(compression());
} else {
    app.use(logger('dev'));
}

app.use(express.json());
app.use(express.urlencoded({extended : false}));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));

// set routes
//
app.use('/', indexRouter);
app.use('/valence', valenceRouter);

// catch 404 and forward to error handler
//
app.use(function(req, res, next) {
    next(createError(404));
});

// error handler
//
app.use(function(err, req, res, next) {
    // set locals, only providing error in development
    //
    res.locals.message = err.message;
    res.locals.error   = req.app.get('env') === 'development' ? err : {};

    // render the error page
    //
    res.status(err.status || 500);
    res.render('error');
});

module.exports = app;
